@@ -1,13 +1,12 @@
 #
 # This file is part of pyasn1 software.
 #
-# Copyright (c) 2005-2019, Ilya Etingof <etingof@gmail.com>
-# License: http://snmplabs.com/pyasn1/license.html
+# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
+# License: https://pyasn1.readthedocs.io/en/latest/license.html
 #
 import sys
 
 from pyasn1 import error
-from pyasn1.compat import calling
 from pyasn1.type import constraint
 from pyasn1.type import tag
 from pyasn1.type import tagmap
@@ -179,31 +178,31 @@ class NoValue(object):
     Any operation attempted on the *noValue* object will raise the
     *PyAsn1Error* exception.
     """
-    skipMethods = set(
-        ('__slots__',
-         # attributes
-         '__getattribute__',
-         '__getattr__',
-         '__setattr__',
-         '__delattr__',
-         # class instance
-         '__class__',
-         '__init__',
-         '__del__',
-         '__new__',
-         '__repr__',
-         '__qualname__',
-         '__objclass__',
-         'im_class',
-         '__sizeof__',
-         # pickle protocol
-         '__reduce__',
-         '__reduce_ex__',
-         '__getnewargs__',
-         '__getinitargs__',
-         '__getstate__',
-         '__setstate__')
-    )
+    skipMethods = {
+        '__slots__',
+        # attributes
+        '__getattribute__',
+        '__getattr__',
+        '__setattr__',
+        '__delattr__',
+        # class instance
+        '__class__',
+        '__init__',
+        '__del__',
+        '__new__',
+        '__repr__',
+        '__qualname__',
+        '__objclass__',
+        'im_class',
+        '__sizeof__',
+        # pickle protocol
+        '__reduce__',
+        '__reduce_ex__',
+        '__getnewargs__',
+        '__getinitargs__',
+        '__getstate__',
+        '__setstate__',
+    }
 
     _instance = None
 
@@ -220,7 +219,7 @@ class NoValue(object):
                         if (name not in cls.skipMethods and
                             name.startswith('__') and
                             name.endswith('__') and
-                            calling.callable(getattr(typ, name)))]
+                            callable(getattr(typ, name)))]
 
             for name in set(op_names):
                 setattr(cls, name, getPlug(name))
@@ -498,17 +497,39 @@ class ConstructedAsn1Type(Asn1Type):
     strictConstraints = False
 
     componentType = None
-    sizeSpec = None
+
+    # backward compatibility, unused
+    sizeSpec = constraint.ConstraintsIntersection()
 
     def __init__(self, **kwargs):
         readOnly = {
             'componentType': self.componentType,
+            # backward compatibility, unused
             'sizeSpec': self.sizeSpec
         }
+
+        # backward compatibility: preserve legacy sizeSpec support
+        kwargs = self._moveSizeSpec(**kwargs)
+
         readOnly.update(kwargs)
 
         Asn1Type.__init__(self, **readOnly)
 
+    def _moveSizeSpec(self, **kwargs):
+        # backward compatibility, unused
+        sizeSpec = kwargs.pop('sizeSpec', self.sizeSpec)
+        if sizeSpec:
+            subtypeSpec = kwargs.pop('subtypeSpec', self.subtypeSpec)
+            if subtypeSpec:
+                subtypeSpec = sizeSpec
+
+            else:
+                subtypeSpec += sizeSpec
+
+            kwargs['subtypeSpec'] = subtypeSpec
+
+        return kwargs
+
     def __repr__(self):
         representation = '%s %s object' % (
             self.__class__.__name__, self.isValue and 'value' or 'schema'
@@ -655,9 +676,6 @@ class ConstructedAsn1Type(Asn1Type):
 
         return clone
 
-    def verifySizeSpec(self):
-        self.sizeSpec(self)
-
     def getComponentByPosition(self, idx):
         raise error.PyAsn1Error('Method not implemented')
 
@@ -679,5 +697,10 @@ class ConstructedAsn1Type(Asn1Type):
     def getComponentType(self):
         return self.componentType
 
-# Backward compatibility
+    # backward compatibility, unused
+    def verifySizeSpec(self):
+        self.subtypeSpec(self)
+
+
+        # Backward compatibility
 AbstractConstructedAsn1Item = ConstructedAsn1Type
